#! /usr/bin/perl
#
# wanha - your pesky URL-guardian for IRC
# Copyright (C) 2006 Tommi Saviranta  <wnd@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

use strict;
use warnings FATAL => 'all';
use diagnostics;

require "html.pl";
require "http.pl";
require "db.pl";
require "dbops.pl";
require "url.pl";
require "common.pl";
require "lang.pl";
require "urllist.pl";
require "session.pl";
require "op_chan.pl";

my @links;

&main();
exit(0);


sub main()
{
	my @links;

	my $chan = &HTTP::get_numeric("chan", 1);
	my $seek = &HTTP::get_numeric("seek", -1);
	my $n = &HTTP::get_numeric("n", -1);

	my @extra;
	my $line = sprintf("<link rel=\"alternate\" " .
			"href=\"%s\" " .
			"type=\"application/atom+xml\" " .
			"title=\"ATOM - wanha, channel %s\" />",
			&URL::atom_chan($chan),
			&Chan::get_name($chan));
	push(@extra, $line);
	&HTTP::print_header();
	&HTML::print_header(undef, @extra);

	&Session::save_channel(&Session::get_session(), $chan);
	&Session::print_links($chan, &URL::chan($chan, $n, $seek));

	if ($seek == -1) {
		&summary($chan);
	} else {
		&url_hist($chan, $n, $seek);
	}

	&Common::links_add(&URL::chanstat($chan), &Lang::get("link-chanstat"));
	&Common::links_add(&URL::stat($chan), &Lang::get("link-search"));

	&Common::print_links(@links);

	&HTML::print_footer(&DB::process_time());
}


# 0: chan
sub summary($$)
{
	my ($id) = @_;

	my $chan = &Chan::get_name($id);
	(defined($chan)) || &HTML::bad_param("bad chan");

	printf "<h1>" . &Lang::get("chan-summary") . "</h1>\n", $chan;

	&combined_top($id, $chan, 5);
	&urls($id, $chan, 10);
}


# 0: chan
# 1: n
# 2: seek
sub url_hist($$$)
{
	my ($id, $n, $seek) = @_;

	my $chan = &Chan::get_name($id);
	(defined($chan)) || &HTML::bad_param("bad chan");

	printf "<h1>" . &Lang::get("chan-hist-for") . "</h1>\n", $chan;

	&urls($id, $chan, $n, $seek);
}


# 0: chan id
# 1: chan name
# 2: n
sub combined_top($$$)
{
	my ($id, $name, $n) = @_;

	my @fame = &get_top($id, $n, 1);
	my @shame = &get_top($id, $n, 0);
	my @paster = &get_pasters($id, $n);

	($#fame == -1 && $#shame == -1 && $#paster == -1) && return;

	my $l_rank = &Lang::get("col-rank");
	my $l_nick = &Lang::get("col-nick");
	my $l_points = &Lang::get("col-points");
	my $l_urls = &Lang::get("col-urls");
	my $l_point_p = &Lang::get("col-points-ratio");
	my $l_fame = &Lang::get("col-fame");
	my $l_shame = &Lang::get("col-shame");
	my $l_busy = &Lang::get("col-busy");

	print <<EOF;
<table class="wall">
	<colgroup span="2" class="fame" />
	<colgroup class="shared" />
	<colgroup span="2" class="shame" />
	<colgroup class="shared" />
	<colgroup span="4" class="paster" />

	<tr>
		<th colspan="2">$l_fame</th>
		<th class="srank"></th>
		<th colspan="2">$l_shame</th>
		<th class="srank"></th>
		<th colspan="4">$l_busy</th>

	</tr>
	<tr class="ul">
		<th>$l_nick</th>
		<th>$l_points</th>
		<th class="srank">$l_rank</th>
		<th>$l_nick</th>
		<th>$l_points</th>
		<th class="srank">$l_rank</th>
		<th>$l_nick</th>
		<th>$l_urls</th>
		<th>$l_points</th>
		<th>$l_point_p</th>
	</tr>
EOF

	my $rn = &Common::max(&Common::max($#fame, $#shame), $#paster) + 1;
	for (my $i = 0; $i < $rn; $i++) {
		my $class = $i % 2 == 1 ? " class=\"zebra\"" : "";
		print "\t<tr$class>\n";

		&print_fame_shame($fame[$i]);
		print "\t\t<td class=\"srank\">#" . ($i + 1) . "</td>\n";
		&print_fame_shame($shame[$i]);
		print "\t\t<td class=\"srank\">#" . ($i + 1) . "</td>\n";
		&print_paster($paster[$i]);

		print "\t</tr>\n";
	}

	print "</table>\n";
}


# 0: hash
sub print_fame_shame(%)
{
	my %t = %{(shift)};
	if (! defined($t{'id'})) {
		print "\t\t<td colspan=\"2\"></td>\n";
		return;
	}

	my $a_nick = &URL::user($t{'id'});
	my $p_nick = &HTML::htmlize($t{'name'});
	my $p_score = $t{'score'};
	print <<EOF;
		<td><a href="$a_nick">$p_nick</a></td>
		<td class="center">$p_score</td>
EOF
}


# 0: hash
sub print_paster($)
{
	my %t = %{(shift)};
	if (! defined($t{'id'})) {
		print "\t\t<td colspan=\"4\"></td>\n";
		return;
	}

	my $a_nick = &URL::user($t{'id'});
	my $p_nick = &HTML::htmlize($t{'name'});
	my $p_count = $t{'url_count'};
	my $p_fame = $t{'fame'};
	my $p_hit_r = sprintf("%.1f", $t{'fame'} * 100 / $t{'url_count'});

	print <<EOF;
		<td><a href="$a_nick">$p_nick</a></td>
		<td class="center">$p_count</td>
		<td class="center">$p_fame</td>
		<td class="center">$p_hit_r%</td>
EOF
}


# 0: chan id
# 1: n
# 2: fame
# r: array of hash (id, name, score)
sub get_top($$$)
{
	my ($chan_id, $n, $fame) = @_;
	my ($query, $result);

	my $gcol = $fame == 1 ? "fame" : "shame";
	my $bcol = $fame == 1 ? "shame" : "fame";
	$query = "SELECT nicks.id, nicks.nick, scoreboard.$gcol " .
			"FROM nicks, scoreboard " .
			"WHERE scoreboard.chan = $chan_id " .
				"AND scoreboard.nick = nicks.id " .
			"ORDER BY scoreboard.$gcol DESC, " .
				"scoreboard.$bcol ASC " .
			"LIMIT $n";
	$result = &DB::query($query);
	($result->ntuples == 0) && return ();

	my @ret;
	while (my ($id, $name, $score) = $result->fetchrow) {
		my %t = (
			'id' => $id,
			'name' => $name,
			'score' => $score
			);
		push(@ret, \%t);
	}
	return @ret;
}


# 0: chan id
# 1: chan name
# 2: n
# 3: seek
sub urls($$$$)
{
	my ($chan_id, $chan_name, $n, $seek) = @_;
	my @urls;
	my @images;
	my $has_more;

	$n = 50 if ($n > 50);

	# get one extra so that we know if there's more
	@urls = &get_urls($chan_id, $n + 1, $seek);
	($#urls == -1) && return;
	if ($#urls == $n) {
		$has_more = 1;
		pop @urls;
	} else {
		$has_more = 0;
	}

	if (! defined($seek)) {
		printf "<h2>" . &Lang::get("chan-last-n-urls") . "</h2>\n", $n;
	} else {
		my $first = $urls[0]{'url_id'};
		my $last = $urls[$#urls]{'url_id'};
		printf "<h2>" . &Lang::get("chan-urls-range") . "</h2>\n",
				$first, $last;
	}

	@images = &URLList::print(\@urls, "url", "nick", "date", "count");

	if (! defined($seek) && $has_more == 1) {
		&Common::links_add(&URL::chan($chan_id, 25, $n),
				&Lang::get("link-older-urls"));
	} elsif (defined($seek)) {
		my $a_fresh = &URL::chan($chan_id);

		&Common::links_add(&URL::chan($chan_id),
				&Lang::get("link-latest-urls"));

		if ($seek > 0) {
			my $tseek = $seek - $n;
			$tseek = 0 if ($tseek < 0);
			&Common::links_add(&URL::chan($chan_id, $n, $tseek),
					&Lang::get("link-newer"));
		}
		if ($has_more == 1) {
			my $tseek = $seek + $n;
			&Common::links_add(&URL::chan($chan_id, $n, $tseek),
					&Lang::get("link-older"));
		}
	}

	&Common::link_images(@images);
}


# 0: chan id
# 1: n
# 2: seek
# r: array of hash (url_id, url_url, url_date, url_count, nick_id, nick_name)
sub get_urls($$$)
{
	my ($chan_id, $n, $seek) = @_;
	my ($query, $result);
	my @ret;

	$query = "SELECT urls.id, urls.url, " .
				&DBOps::get_date_fmt("urls.time") . ", " .
				"urls.count, nicks.nick, nicks.id " .
			"FROM urls, nicks " .
			"WHERE urls.chan = $chan_id AND urls.nick = nicks.id " .
			"ORDER BY urls.id DESC " .
			"LIMIT $n";
	$query .= " OFFSET $seek" if (defined($seek));

	$result = &DB::query($query);
	($result->ntuples == 0) && return ();

	while (my ($url_id, $url_url, $url_date, $url_count,
				$nick_name, $nick_id) =
			$result->fetchrow) {
		my %t = (
			'url_id' => $url_id,
			'url_url' => $url_url,
			'url_count' => $url_count,
			'url_date' => $url_date,
			'nick_id' => $nick_id,
			'nick_name' => $nick_name
			);
		push(@ret, \%t);
	}
	return @ret;
}


# 0: chan id
# 1: n
# r: array of hash (id, nick, url_count, fame)
sub get_pasters($$)
{
	my ($chan_id, $n) = @_;
	my ($query, $result);
	my @ret;

	my $subq = "SELECT nicks.id, nicks.nick, SUM(urls.count) AS sum " .
			"FROM nicks, urls " .
			"WHERE nicks.id = urls.nick AND urls.chan = $chan_id " .
			"GROUP BY nicks.id, nicks.nick";
	$query = "SELECT sum.id, sum.nick, sum.sum, score.fame " .
			"FROM ($subq) AS sum, scoreboard AS score " .
			"WHERE sum.id = score.nick AND score.chan = $chan_id " .
			"ORDER BY sum DESC LIMIT $n";
	$result = &DB::query($query);
	($result->ntuples == 0) && return ();

	while (my ($nick_id, $nick_name, $url_count, $fame) =
			$result->fetchrow) {
		my %t = (
			'id' => $nick_id,
			'name' => $nick_name,
			'url_count' => $url_count,
			'fame' => $fame
			);
		push(@ret, \%t);
	}

	return @ret;
}
